/* -*- Mode: C++ -*-
 * $Id: FCFooCoachDecision.cc,v 1.1 2001/11/15 16:50:13 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFooCoachDecision.cc
 * Unitname: Framework/example/FCFooCoach
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2001-11-15
 * Last modified by $Author: frehe $ $Date: 2001/11/15 16:50:13 $
 *
 *
 * HISTORY
 *
 * $Log: FCFooCoachDecision.cc,v $
 * Revision 1.1  2001/11/15 16:50:13  frehe
 * Added a coach agent
 *
 */

/**
 * @file FCFooCoachDecision.cc
 * Implementation of the class FCFooCoachDecision.
 *
 * @par Unit:
 *      Framework/example/FCFooCoach
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2001/11/15 16:50:13 $
 */


#include <unistd.h>
#include <errno.h>

#include "FCFooCoachDecision.h"

#include "ViewManager.h"
#include "ParameterView.h"
#include "CoachView.h"
#include "view_constants.h"
#include "ActuatorInterface.h"
#include "InitCommand.h"
#include "CoachCommand.h"
#include "rs_assert.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE


FCFooCoachDecision::FCFooCoachDecision(ActuatorInterface* const actuator_interface)
  : Decision(actuator_interface),
    viewManager(ViewManager::instance()),
    parameterView(NULL),
    coachView(NULL),
    serverAlive(true),
    aural_data(0)
#ifdef RS_USE_DISPLAY
  , theDisplay()
#endif
#ifdef RS_USE_OWN_LOGFILE
  , logFile("FCFooCoach.log")
#endif
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(coachView, COACH_VIEW_ID);
}


FCFooCoachDecision::~FCFooCoachDecision()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(COACH_VIEW_ID);
}


ServerCommand* FCFooCoachDecision::initAgent()
{
  RS_REQUIRE( parameterView != NULL );
  InitCommand* init_cmd = new InitCommand(parameterView->CP_team_name,
					  parameterView->SP_version,
					  false);
  return init_cmd;
}


bool FCFooCoachDecision::start()
{
  try {
    while ( serverAlive )
      try {
	if ( pause() == -1 && errno != EINTR )
	  RS_ERROR("Something went wrong in pause!");
      } catch (RSException &excpt) {
	std::cerr << "Caught a RoboSoc exception: " << excpt.What() << std::endl;
	return 1;
      } catch (std::exception &excpt) {
	std::cerr << "Caught a standard exception: " << excpt.what() << std::endl;
	return 1;
      } catch (...) {
	std::cerr << "Caught an unknown exception: " << std::endl;
	return 1;
      }
    return true;
  } catch (RSException &excpt) {
    std::cerr << "Caught a RoboSoc exception: " << excpt.What() << std::endl;
    return 1;
  } catch (std::exception &excpt) {
    std::cerr << "Caught a standard exception: " << excpt.what() << std::endl;
    return 1;
  } catch (...) {
    std::cerr << "Caught an unknown exception: " << std::endl;
    return 1;
  }
}


bool FCFooCoachDecision::decide()
{
  static int say_counter = 10;
  
  if ( say_counter == 0 ) {
    actuatorInterface->addToPlan(&CoachCommand(std::string("(freeform \"I am the coach\")")));
    say_counter = 9;
  } else {
    --say_counter;
  }
  
  return true;
}


/*
void FCFooCoachDecision::onActuatorSensorData()
{
#ifdef RS_USE_DISPLAY
  theDisplay.setLastSkill(commandView->getLastServerString());
#endif
#ifdef RS_USE_OWN_LOGFILE
  logFile << gameView->getCurrentTime() << ": executed action "
	  << commandView->getLastServerString()
	  << std::endl;
  if ( commandView->getActualTurn(NOW) > AngleDeg(0) )
    logFile << gameView->getCurrentTime() << ": actual turn = "
	    << commandView->getActualTurn(NOW)
	    << std::endl;  
#endif
}


void
FCFooCoachDecision::onAuralCoachSensorData()
{
  std::cerr << "I got " << agentView->getLastCoachMessage()
	    << " from the coach" << std::endl;
}


void
FCFooCoachDecision::onAuralRefereeSensorData()
{
  std::cerr << "I got message from the referee." << std::endl;
}
*/

void
FCFooCoachDecision::onAuralPlayerSensorData()
{
#ifdef RS_USE_DISPLAY
  theDisplay.draw();
#endif
}

/*
void
FCFooCoachDecision::onAuralSelfSensorData()
{
  std::cerr << "I got " << agentView->getLastSelfMessage()
	    << " from myself" << std::endl;
  ++aural_data;
}
*/

void FCFooCoachDecision::onInit()
{
  actuatorInterface->addToPlan(&CoachCommand(std::string("(eye on)")));

#ifdef RS_USE_DISPLAY
  theDisplay.setLastSkill(std::string("(eye on)"));
  theDisplay.draw();
#endif
}


void FCFooCoachDecision::onVisualSensorData()
{
  //RS_LOG( LA_DECISION, gameView->getCurrentTime() << ": **saw**" );
#ifdef RS_USE_DISPLAY
  theDisplay.draw();
#endif
}

/*
void FCFooCoachDecision::onAfterSensorData()
{
}
*/


void FCFooCoachDecision::onNewCycle(const bool forced)
{
  if ( forced ) {
    RS_LOG( LA_DECISION, "**Forced NewCycle**" );
  }
  else {
    RS_LOG( LA_DECISION, "**NewCycle**" );
  }

#ifdef RS_USE_DISPLAY
  theDisplay.draw();
#endif

}


/*
bool FCFooCoachDecision::onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks)
{
  // We assume that if the sense_body is more than four ticks late
  // then it's lost.
  RS_LOG( LA_DECISION, "onEstimatedNewCycle(" << estimated_time
	  << ", " << ticks << ")" );

  // If more than nine sense_body's are missed in a row then assume that
  // the server is dead.
  if ( estimated_time > 9 )
    serverAlive = false;

  if ( ticks > 4 ) {
    // TODO: change this into sending a control command instead!
    //sensorInterface->ForceNewCycle();
    return true;
  } else
    return false;
}
*/


void FCFooCoachDecision::onCommandWarning()
{
  decide();
}


RS_END_NAMESPACE
