/* -*- Mode: C++ -*-
 * $Id: IsOffside.cc,v 1.4 2001/10/03 19:43:39 aip5 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsOffside.cc
 * Unitname: Framework/predicate
 * $Revision: 1.4 $
 * Created by: David Rosen 2001-09-30
 * Last modified by $Author: aip5 $ $Date: 2001/10/03 19:43:39 $
 *
 *
 * HISTORY
 *
 *
 */

/**
 * @file IsOffside.cc
 * Implementation file for the predicate class IsOffside.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.4 $
 * $Author: aip5 $
 * $Date: 2001/10/03 19:43:39 $
 */


#include "IsOffside.h"

#include "WorldStateView.h"
#include "ParameterView.h"
#include "view_constants.h"



RS_BEGIN_NAMESPACE


IsOffside::IsOffside(int teamname, const PlayerState &playerstate, 
		     bool assumeGoalieIsNearGoal)
  : Predicate(),
    worldStateView(NULL),
    parameterView(NULL),
    team(teamname),
    player(playerstate),
    goalieNearGoal(assumeGoalieIsNearGoal) {
  getView(worldStateView, WORLD_STATE_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
}


IsOffside::IsOffside(const IsOffside& obj)
  : Predicate(obj),
    worldStateView(obj.worldStateView),
    parameterView(obj.parameterView),
    team(obj.team),
    player(obj.player),
    goalieNearGoal(obj.goalieNearGoal) {
}


IsOffside::~IsOffside() {
  viewManager->releaseView(worldStateView->getId());
  viewManager->releaseView(parameterView->getId());
}


IsOffside& 
IsOffside::operator= (const IsOffside& obj) {
  if ( this != &obj ) {
    Predicate::operator=(obj);
    worldStateView = obj.worldStateView;
    parameterView = obj.parameterView;
    team = obj.team;
    player = obj.player;
    goalieNearGoal = obj.goalieNearGoal;
  }
  return *this;
}

FuzzyBool 
IsOffside::operator()() {
  return evaluate();
}


FuzzyBool 
IsOffside::evaluate() {
  double offsideConf = 1.0;

  double offsideLine = 0;
  double offsideLineConf = 1.0;

  double lastDefender = 0;
  double lastDConf = 1.0;
  double secondlastDefender=0;
  double secondDConf = 1.0;    
  int m;

  std::vector<PlayerState> op;
  if (team == T_OUR_TEAM) {
    op = worldStateView->getWorldState().getOpponents();
    m = 1;
//      std::cout << "---- Seeing " << op.size() << " opponents ----" << std::endl;
//      std::cout << "Offsideline for OUR   team: ";
  }
  else if (team == T_THEIR_TEAM) {
    op = worldStateView->getWorldState().getTeammates();
    // lgg till mig sjlv
    op.push_back(worldStateView->getWorldState().getAgent());
    // Inte s intuitiv kod kanske, men fr att slippa
    // skriva allt igen, vi vnder p alla positioner.
    // och rknar som det vore vr sidas offside.
    m = -1;
//      std::cout << "---- Seeing " << op.size() << " teammates ----" << std::endl;
//      std::cout << "Offsideline for THEIR team: ";
  }
  else {
    std::cout << "ErrorError - frn IsOffside" << std::endl;
    return FuzzyBool(0.0);
  }


  std::vector<PlayerState>::const_iterator plr;
  for(plr = op.begin(); plr != op.end(); plr++) {
    double x = m*plr->getPosition().getX();
    double conf = plr->getC();
    if(x > secondlastDefender) 
      if (x > lastDefender) {
	secondlastDefender = lastDefender;
	secondDConf = lastDConf;
	lastDefender=x;
	lastDConf = conf;
      }
      else {
	secondlastDefender = x;
	secondDConf = conf;
      }      
  }
  // Vi antar att mlvakten r nrheten av mlet, och om vi inte sett honom
  // s tar vi det som ett faktum att han r drngonstans.
  if ((goalieNearGoal) && (lastDefender < 40)) {// Han springer inte ut lngre? 
    // Vi anser oss ha missat att se mlvakten och plockar drfr
    // sista sedda spelare som offsideupphvare.
    secondlastDefender = lastDefender;
    secondDConf = lastDConf;
  }
  
  offsideLine = secondlastDefender;
  offsideLineConf = secondDConf;
  double plx = m*player.getPosition().getX();
  double plc = player.getC();
  // Spelaren r offside, hur mycket kan vi lita p informationen?
  // M-x ta-saker-helt-ur-luften-mode
  // Med noll konfidens p spelare, och linje s litar vi p det
  // till 100% om han r mer n 2 bakom linjen 
  double const safed = 2.0;
  
  double worstplx = plx - (1.0 - plc)*safed;
  if (worstplx < 0) // Fel planhalva
      offsideConf = 0.0;
  else {
    double worstline = offsideLine + (1.0 - offsideLineConf)*safed;
    
    // r detta strre n 0 anser vi oss helt skra p att
    // spelaren r offside.
    double distance = worstplx - worstline;
    // Annars s rknas confidensen p hur mycket detta skiljer sig
    offsideConf = max (min (1.0, 1.0+distance/safed),0.0);
  }
//    std::cout << m*offsideLine << " -- (" << offsideConf << ")" << std::endl;
  return FuzzyBool(offsideConf);
}

void
IsOffside::update(const UInt32) {}
		  

RS_END_NAMESPACE
