/* -*- Mode: C++ -*-
 * $Id: FFClearBall.h,v 1.1 2001/09/18 20:42:31 andbo928 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FFClearBall.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: andbo928 $ $Date: 2001/09/18 20:42:31 $
 *
 *
 * HISTORY
 * $Log: FFClearBall.h,v $
 * Revision 1.1  2001/09/18 20:42:31  andbo928
 * Imported some sources from FuzzyFoo.
 *
 * Revision 1.4  2001/07/29 14:10:00  andbo928
 * Now uses AngleThingy class.
 *
 * Revision 1.3  2001/07/16 15:35:39  andbo928
 * Fixed some bugs.
 *
 * Revision 1.2  2001/04/22 14:13:51  andbo928
 * This one should be fixed.
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.8  2000/10/16 12:27:01  govme133
 * *** empty log message ***
 *
 * Revision 1.7  2000/10/09 14:06:49  govme133
 * final version
 *
 * Revision 1.6  2000/10/07 14:35:55  govme133
 * version using the angles between opponents
 *
 * Revision 1.5  2000/09/27 13:39:05  govme133
 * nearly finished skill
 *
 * Revision 1.4  2000/09/24 20:21:31  govme133
 * *** empty log message ***
 *
 * Revision 1.3  2000/09/24 19:13:39  govme133
 * *** empty log message ***
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

/**
 * @file FFClearBall.h
 * Include file for the skill class FFClearBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: andbo928 $
 * $Date: 2001/09/18 20:42:31 $
 */


#ifndef FFCLEARBALL_H
#define FFCLEARBALL_H


#include "Skill.h"
#include "WorldStateView.h"
#include <list>

RS_BEGIN_NAMESPACE

/**
 * @class FFClearBall FFClearBall.h
 * This class encapsulates a FFClearBall skill.
 * It's called in an emergency (probably in front of the own goal) 
 * and it results in kicking the ball to an open area on 
 * the field, preferably towards the enemy side of the field.
 *
 * <!--
 * @pre The preconditions for the skill are...
 *  The player needs to be in kicking distance to the ball
 *  It is impossible to give a good pass to a friendly player
 *  It is to dangerous to try to dribble (e.g. if the enemy players get the ball they will score)
 * @post The postconditions for the skill are...
 *  The ball is kicked to an empty area on the field if an appropriate area exists.
 *  if no good area exists within the view of the player he tries to kick it in the direction 
 *  of the enemy goal.
 *  The ball is NEVER kicked in the direction of the own side of the field (because this could 
 *  cause a more dangerous situation).
 *  The ball is only kicked outside of the field if there is no better solution.
 * @invariant The class invariant is...
 *   what is an invariant :)
 * @todo This needs to be done...
 * --> nothing
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: andbo928 $
 * $Date: 2001/09/18 20:42:31 $
 */

class FFClearBall : public Skill {
public:
  FFClearBall();
  virtual ~FFClearBall();
  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

protected:
//    struct Span {
//      double start;
//      double end;
//    };

  WorldStateView* worldView;
  ParameterView* parameterView;
};


RS_END_NAMESPACE


#endif // !FFCLEARBALL_H
