/* -*- Mode: C++ -*-
 * $Id: KickBallToAbsoluteSpeedVector.h,v 1.3 2001/09/01 23:51:46 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBall.h
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/09/01 23:51:46 $
 *
 *
 * HISTORY
 * $Log: KickBallToAbsoluteSpeedVector.h,v $
 * Revision 1.3  2001/09/01 23:51:46  frehe
 * Updated the skill with the new abstract skill class and the new version of getValue and getAngle.
 *
 * Revision 1.2  2001/08/23 00:31:58  frehe
 * Removed dependencies.
 *
 * Revision 1.1  2001/01/23 17:18:17  olalu526
 * Added kick to absoulute vector
 *
 */

#ifndef KICKBALLTOABSOLUTESPEEDVECTOR_H
#define KICKBALLTOABSOLUTESPEEDVECTOR_H


#include "SkillWithVector.h"
#include "KickBallToSpeedVector.h"


RS_BEGIN_NAMESPACE


class ParameterView;
class AgentView;


/**
 * @class KickBallToAbsoluteSpeedVector KickBallToAbsoluteSpeedVector.h
 * This class encapsulates a kicking skill.
 *
 * 
 * @pre The preconditions for the skill are:
 *  Same as the relative kick skill + that the agent has to know where
 *  it is.
 *
 * @post The postconditions for the skill are...
 * 
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:51:46 $
 */
class KickBallToAbsoluteSpeedVector : public SkillWithVector {
public:
  KickBallToAbsoluteSpeedVector(const Vector& vec);
  virtual ~KickBallToAbsoluteSpeedVector();

  virtual Plan generatePlan();
  virtual Plan generatePlan(const Vector& vec);

  virtual void setVector(const Vector& vec);
  virtual Vector getVector() const;

  virtual FuzzyBool persistent();

  virtual FuzzyBool applicable();
  virtual FuzzyBool applicable(const Vector& vec);

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };

  
protected:
  Vector vect;
  KickBallToSpeedVector kick;
  ParameterView* parameterView;
  AgentView* agentView;
};


RS_END_NAMESPACE


#endif // !KICKBALLTOABSOLUTESPEEDVECTOR_H


