/* -*- Mode: C++ -*-
 * $Id: MarkPlayerFSAL.cc,v 1.4 2001/10/09 18:50:08 aip4 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MarkPlayerFSAL.cc
 * Unitname: Framework/skill
 * $Revision: 1.4 $
 * Created by: Anders Lindahl 2001-09-29
 * Last modified by $Author: aip4 $ $Date: 2001/10/09 18:50:08 $
 *
 *
 * HISTORY
 *
 * $Log: MarkPlayerFSAL.cc,v $
 * Revision 1.4  2001/10/09 18:50:08  aip4
 * Added documentation
 *
 * Revision 1.3  2001/10/09 18:16:03  aip4
 * Seem to work as planned now
 *
 * Revision 1.2  2001/09/29 18:55:21  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.1  2001/09/29 12:31:40  aip4
 * Added MarkPlayerFSAL skill.
 *
 */

/**
 * @file MarkPlayerFSAL.cc
 * Implementation file for the skill class MarkPlayerFSAL.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: aip4 $
 * $Date: 2001/10/09 18:50:08 $
 */

#include "MarkPlayerFSAL.h"
#include "MoveToPositionAvoidingObstacles.h"

RS_BEGIN_NAMESPACE

// --- CONSTRUCTORS ---

MarkPlayerFSAL::MarkPlayerFSAL()
  : worldStateView(NULL),
    parameterView(NULL),
    agentView(NULL),
    ballView(NULL),
    playerView(NULL)
{
  
  markedOpponent = NULL;
  ballMemory = 10;
  opponentMemory = 10;

  getView(worldStateView, WORLD_STATE_VIEW_ID);
  getView(parameterView,PARAMETER_VIEW_ID);
  getView(agentView,AGENT_VIEW_ID, parameterView);
  getView(ballView,BALL_VIEW_ID);
  getView(playerView,PLAYER_VIEW_ID);
}


// --- DESTRUCTOR ---
MarkPlayerFSAL::~MarkPlayerFSAL()
{
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(WORLD_STATE_VIEW_ID);

}


// --- METHODS ---


/** Finds the closest unmarked opponent (if we haven't done this previously) 
 *  and mark it.
 */
Plan MarkPlayerFSAL::generatePlan()
{
  int gameTime = worldStateView->getWorldState().getGameTime();
  static MoveToPositionAvoidingObstacles moveToPositionAvoidingObstacles;
  static Command* cmd;
  static PointUnknown agentPosition;
  static AngleDegUnknown toDirection, bodyDirection;
  static AngleDeg angle;
  static FloatUnknown distanceToPosition;
  static Plan plan;
  VectorUCO opponentToBall;
  PointUCO moveToPoint;  

  opponents = playerView->getOpponents(NOW);

  // Maybe we should check confidence instead?
  if( ballView->getBallPosition(NOW).isKnown() ) {
    ballLastPosition = ballView->getBallPosition(NOW);
    ballTime = gameTime; //Spara tiden fr senare jmfrelser.
  }      
  
  if( markedOpponent == NULL) {
    // We haven't choosen an opponent to mark yet. Allocate one.
    markedOpponent = new PlayerObject();
    PlayerObject* dummy = getClosestUnmarkedOpponent();
    if (dummy != NULL) {
      *markedOpponent = *dummy;
      opponentTime = gameTime;
    }
    else {
      // Shit, we didn't find an unmarked opponent. This should be avoided
      // in applicable()
      delete markedOpponent;
      markedOpponent = NULL;
      opponentTime=0;
      return Plan();
    }
  }
  
  for (unsigned int i=0;i < opponents.size();i++) {
    if ( opponents[i].getUniformNumber().isKnown() )
      if (opponents[i].getUniformNumber().getValue() == markedOpponent->getUniformNumber())
	if (opponents[i].getPosition().isKnown() ) {
	  // We have found our player on the field, and its position is know. Save for later.
	  *markedOpponent = opponents[i];
	  opponentTime=gameTime;
	  break;
	}
    }

  if (gameTime - opponentTime > opponentMemory) {
    // We don't know the opponents position well enough. Turn to look for it.
    RS_LOG(LA_TOP,"MARKPLAYERFSAL: Letar efter motstnaden.");
    agentPosition = agentView->getAgentPosition(NOW);
    toDirection = agentPosition.getDirectionTo(markedOpponent->getPosition());
    bodyDirection = agentView->getBodyDirection(NOW);
    cmd = new TurnCommand(toDirection.getAngle() - bodyDirection.getAngle());
  }
  else if ((gameTime - ballTime > ballMemory) &&
	   (gameTime - ballTime < (ballMemory * 2))) {
    // The ball position has been unknown for a while, look for it.
    // if the ball has been gone for to long, forget it and run
    // towards the opponent instead.
    agentPosition = agentView->getAgentPosition(NOW);
    toDirection = agentPosition.getDirectionTo(ballLastPosition);
    bodyDirection = agentView->getBodyDirection(NOW);
    cmd = new TurnCommand(toDirection.getAngle() - bodyDirection.getAngle()); 
  }
  
  else {
    // We are happy with our knowledge about the opponent and ball.
    // or, at least with the opponent.
    if (gameTime - ballTime >= (ballMemory * 2))
      // The ball position is unknown, stay close to the opponent.
      moveToPoint = markedOpponent->getPosition();
    else{
      //calculate the point we should move to
      opponentToBall = markedOpponent->getRelativeVectorTo(ballLastPosition);
      opponentToBall *= 0.25;
      moveToPoint = markedOpponent->getPosition() + opponentToBall;
    }
    if ( moveToPoint.isKnown() )
      plan = moveToPositionAvoidingObstacles.generatePlan( moveToPoint.getPoint() );
    else {
      // If the calculated point for some reason is unknown, extrapolate the position
      // of the opponent and run towards it.
      PointUCO tmpPoint = markedOpponent->getPosition() + markedOpponent->getAbsoluteSpeedVector();
      plan = moveToPositionAvoidingObstacles.generatePlan( tmpPoint.getPoint() );
    }
  }
  if (cmd == NULL)
    return plan;

  plan = Plan(Step(cmd));
  delete cmd;
  cmd = NULL;
  return plan;
}

/** Not persistent, hence returns FuzzyBool(0.0)
 *
 */
FuzzyBool MarkPlayerFSAL::persistent()
{
  //Never persistent
  return FuzzyBool(0.0);
}

/** Applicable if...
 *  a) We already have a player to mark, and we have seen it recently
 *  b) An unmarked opponent is visible
 */
FuzzyBool MarkPlayerFSAL::applicable()
{
  int gameTime = worldStateView->getWorldState().getGameTime();
  if (markedOpponent != NULL &&
      (gameTime - opponentTime) < (2 * opponentMemory)) {
    // We already have an opponent allocated. 
    return FuzzyBool(1.0);
  }
  
  if (markedOpponent != NULL) {
    // An opponent is choosen, but we haven't seen him (her?) in
    // a while. Forget it.
    delete markedOpponent;
    opponentTime = 0;
    markedOpponent = NULL;
  } 
    
  teammates = playerView->getTeammates(NOW); //a copy of the actual data
  opponents = playerView->getOpponents(NOW); //a copy of the actual data
  
  if ( opponents.size() == 0)
    return FuzzyBool(0.0); // If no visible opponents, no one to mark.
  
  for (unsigned int i=0;i < opponents.size();i++) {
    if (!playerMarked(opponents[i]))
      return FuzzyBool(1.0);
  }
  
  return FuzzyBool(0.0);
}

/** Returns true if the playerobject is marked. It is marked
 * when a teammate is within 10 units from it.
 */
FuzzyBool MarkPlayerFSAL::playerMarked(PlayerObject player) {
  if (player.getPosition().isKnown()) {
    VectorUCO diffVector;
    VectorUCO opponent_to_ball; 
    opponent_to_ball = player.getRelativeVectorTo( ballView->getBallPosition(NOW) );
    double markDistance = 10;
    
    for(unsigned int i = 0; i< teammates.size(); i++) {
      if (teammates[i].getPosition().isKnown()) {
	diffVector = player.getRelativeVectorTo( teammates[i].getPosition() );
	if ( diffVector.getLength() > markDistance ) {
	  return FuzzyBool(1.0);
	}
      }
    }
  }
  return FuzzyBool(0.0);
}


/** Loops over all visible opponents, returns the
 * closest unmarked. (!)
 */
PlayerObject* MarkPlayerFSAL::getClosestUnmarkedOpponent(){
  opponents = playerView->getOpponents(NOW);
  float minDiff = 1000000;
  VectorUCO relativeVector;
  PlayerObject* tempPekare = NULL;

  for ( unsigned int i = 0; i < opponents.size(); i++ ) {
        if (agentView->getAgentPosition(NOW).isKnown())
	if (opponents[i].getPosition().isKnown()) {
      relativeVector = opponents[i].getRelativeVectorTo( agentView->getAgentPosition(NOW) );
      if( minDiff > relativeVector.getLength().getValue() &&
	  !playerMarked( opponents[i] ) ){
	minDiff = relativeVector.getLength().getValue();
	tempPekare = &opponents[i];
      }
    }
  }
  return tempPekare;
}

RS_END_NAMESPACE











