/* -*- Mode: C++ -*-
 * $Id: PassForGoalChance.cc,v 1.7 2001/10/10 05:14:14 aip10 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PassForGoalChance.cc
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Pelle Johansson, Mikael Lindstrm, Per-Ola ... 2001-09-05
 * Last modified by $Author: aip10 $ $Date: 2001/10/10 05:14:14 $
 *
 *
 * HISTORY
 *
 * $Log: PassForGoalChance.cc,v $
 * Revision 1.7  2001/10/10 05:14:14  aip10
 * Smarter CanScore
 *
 * Revision 1.6  2001/10/03 08:55:51  aip10
 * Switched to WorldStateView and added offside check.
 *
 * Revision 1.5  2001/09/29 18:55:22  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.4  2001/09/28 11:45:49  aip10
 * Updated PassForGoalChance
 *
 * Revision 1.3  2001/09/26 11:41:27  frehe
 * Update after move of TRUE, FALSE and UNKNOWN to inside FuzzyBool.
 *
 * Revision 1.2  2001/09/05 14:58:26  peljo105
 * Implemented calculatePassVector() and CanPass()
 *
 * Revision 1.1  2001/09/05 11:41:44  peljo105
 * Frsta version
 *
 *
 */

/**
 * @file PassForGoalChance.cc
 * Implementation file for the skill class PassForGoalChance.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: aip10 $
 * $Date: 2001/10/10 05:14:14 $
 */


#include "PassForGoalChance.h"

#include "view_constants.h"
#include "AbleToKickBallToPoint.h"

#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "PlayerView.h"
#include "IsOffside.h"
#include "AngleThingy.h"
#include "TVector.h"

#include <iostream>

#include <unistd.h>


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

PassForGoalChance::PassForGoalChance()
  : Pass()
{
  getView(markerView, MARKER_VIEW_ID, parameterView);
  getView(worldView, WORLD_STATE_VIEW_ID);
}

// --- DESTRUCTOR ---

PassForGoalChance::~PassForGoalChance()
{
  viewManager->releaseView(MARKER_VIEW_ID);
}


// --- METHODS ---

FuzzyBool PassForGoalChance::applicable()
{
  std::cerr << "Entering applicable()" << std::endl;
  
  if(!Pass::applicable())
  {
    std::cerr << "!Pass::applicable()" << std::endl;
    return FuzzyBool(0.0);
  }
  
#if 0
  // Check if we can shoot for score.
  if(markerView->getTheirGoal(NOW).getPosition().isKnown()) // do we know the goal position ?
  {
    AbleToKickBallToPoint predATKBTP; // predicate
    std::vector<Float> args(2);
    args[0] = markerView->getTheirGoal(NOW).getPosition().getPoint().getX();
    args[1] = markerView->getTheirGoal(NOW).getPosition().getPoint().getY();
    
    predATKBTP.setArguments(args); // update position in predicate
    
    if(predATKBTP.evaluate()>FuzzyBool(0.5))
    {
      std::cerr << "predATKBTP.evaluate()>0.5" << std::endl;
      return FuzzyBool(0.0);
    }
  }
#endif
  
  std::cerr << "appicable() successful" << std::endl;
  return FuzzyBool(true);
}  

Vector PassForGoalChance::calculatePassVector()
{
  std::cerr << "calculatePassVector()" << std::endl;
  // Calculates the speedvector to a desired pass.
  const WorldState &worldState=worldView->getWorldState();
  std::vector<PlayerState> friends=worldState.getTeammates(),
    foes=worldState.getOpponents(),okFriends;
  
  AbleToKickBallToPoint predATKBTP;
  std::vector<Float> args(2);
  
  for(UInt i=0;i<friends.size();i++)
  {
    PlayerState &currPl=friends[i];
    
    TVector vec=currPl.getPosition();
    if(vec.getC()>0.75)
    {
      if(!CanScore(currPl,foes))
      {
	std::cerr << "Teammate " << i << " can't score.\n";
	continue;
      }
      args[0]=vec.getX();
      args[1]=vec.getY();
      predATKBTP.setArguments(args);
      
      if(!predATKBTP.evaluate())
      {
	std::cerr << "Can't pass teammate " << i << std::endl;
	continue;
      }
      
      IsOffside offside(T_OUR_TEAM,currPl);
      
      if(offside.evaluate()>FuzzyBool(0.5))
      {
	std::cerr << "Teammate " << i << " is offside.\n";
	continue;
      }
      
      okFriends.push_back(friends[i]);
    }
  }
  
  if(!okFriends.size())
  {
    std::cerr << "No friends to pass" << std::endl;
    return Vector(0,0); // Noone to pass.
  }
  
  PlayerState *best=NULL;
  for(UInt i=0;i<okFriends.size();i++)
  {
    if(!best || okFriends[i].getC()>best->getC())
      best=&okFriends[i];
  }
  
  TVector passVector=best->getPosition()-worldState.getAgent().getPosition();
  
  std::cerr << passVector << std::endl;
  passVector.rotate(-agentView->getFaceDirection(NOW).getAngle().getDeg());
  
  std::cerr << passVector << std::endl;
  std::cerr << "calculatePassVector() successful" << std::endl;
  
  TVector speedVector = passVector.normalized();

  std::cerr << speedVector << std::endl;
  // This is supposed to create a kick with a limitid velocity
  // at the destination.
  // The speed at the destination should be 0.3 m/s
  speedVector =
    speedVector * 
    (0.3 + passVector.getLength() * (1 - parameterView->SP_ball_decay));
  
  std::cerr << speedVector << std::endl;
  return Vector(speedVector.getAngle(),speedVector.getLength());
}

FuzzyBool PassForGoalChance::CanScore(PlayerState &player,std::vector<PlayerState> &foes)
{
  TVector ourPos=player.getPosition();
  
  TVector leftLimit = 
    TVector(parameterView->SP_pitch_length/2,
	    -0.95*parameterView->SP_goal_width/2)-ourPos;
  TVector rightLimit = 
    TVector(parameterView->SP_pitch_length/2,
	    0.95*parameterView->SP_goal_width/2)-ourPos;
  
  AngleThingy angles;
  AngleThingy::Span span;
  span.start = leftLimit.getAngle();
  span.end = rightLimit.getAngle();
  angles.addSpan(span);
  
  const int enemyRadie=5;
  
  // Not too sofisticated but we need to be fast as well.
  for(UInt i=0;i<foes.size();i++)
  {
    PlayerState &currFoe=foes[i];
    
    TVector theirPos=currFoe.getPosition()-ourPos;
    span.start=theirPos.getAngle()-enemyRadie;
    span.end=theirPos.getAngle()+enemyRadie;
    angles.removeSpan(span);
  }
  
  return angles.getSpans().size()>0;
}

RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "PassForGoalChance.icc"
#endif
// To here if you do not have any inline definitions!
