/* -*- Mode: C++ -*-
 * $Id: PlanContainer.h,v 1.3 2001/09/01 23:51:47 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlanContainer.h
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 *
 * HISTORY
 * $Log: PlanContainer.h,v $
 * Revision 1.3  2001/09/01 23:51:47  frehe
 * Updated the skill with the new abstract skill class and the new version of getValue and getAngle.
 *
 * Revision 1.2  2001/08/23 00:32:01  frehe
 * Removed dependencies.
 *
 * Revision 1.1  2000/10/31 12:48:56  mikbr212
 * created
 *
 */

/**
 * @file PlanContainer.h
 * Include file for the skill class PlanContainer.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:51:47 $
 */


#ifndef PLANCONTAINER_H
#define PLANCONTAINER_H


#include "Skill.h"


RS_BEGIN_NAMESPACE


/**
 * @class PlanContainer PlanContainer.h
 * This class is a skill which encapsulates a plan.
 * 
 * A plan can be set to this skill
 * and then generatePlan will return that plan.
 *
 * <!--
 * @pre The preconditions for the skill are...
 * @post The postconditions for the skill are...
 * @invariant The class invariant is...
 *
 * @todo This needs to be done...
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:51:47 $
 */
class PlanContainer : public Skill {
public:
  PlanContainer();
  virtual ~PlanContainer();

  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };

  
  void setPlan(const Plan& thePlan);
  void setConsumePlan(bool flag);

protected:
  Plan plan;
  bool consumePlan;
};


RS_END_NAMESPACE


#endif // !PLANCONTAINER_H
