/* -*- Mode: C++ -*-
 * $Id: BallView.cc,v 1.12 2001/09/17 22:20:21 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallView.cc
 * Unitname: Framework/view
 * $Revision: 1.12 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/09/17 22:20:21 $
 *
 *
 * HISTORY
 *
 * $Log: BallView.cc,v $
 * Revision 1.12  2001/09/17 22:20:21  frehe
 * Added some functionality to use the realtive speed of the ball from rccparser, but it is not used at the moment.
 *
 * Revision 1.11  2001/09/17 21:27:38  andbo928
 * Removed FutureBallView references.
 *
 * Revision 1.10  2001/09/01 23:49:51  frehe
 * Updated the file for the new implementation of getValue and getAngle.
 *
 * Revision 1.9  2001/08/23 00:32:04  frehe
 * Removed dependencies.
 *
 * Revision 1.8  2001/08/22 21:04:02  frehe
 * Removed an inline statement.
 *
 * Revision 1.7  2001/01/11 22:00:20  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.6  2001/01/10 15:12:59  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.5  2000/10/23 12:06:24  frehe
 * Fixed the ball speed estimation.
 *
 * Revision 1.4  2000/10/11 18:39:11  frehe
 * Changed the ball speed estimation.
 *
 * Revision 1.3  2000/10/08 12:11:04  frehe
 * Fixed a bug in BallView.
 *
 * Revision 1.2  2000/10/04 12:03:15  frehe
 * Made some minor changes.
 *
 * Revision 1.1  2000/09/21 13:46:46  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.3  2000/09/13 00:08:06  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also changed those views that used a static pointer to itself to do process the data in the sensor interface to use the new forEach methods implemented in the FrameworkSensorInterface.
 *
 * Revision 1.2  2000/09/05 21:15:57  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/04 15:27:20  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file BallView.cc
 * Implementation file for the class BallView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.12 $
 * $Author: frehe $
 * $Date: 2001/09/17 22:20:21 $
 */

#include <iostream.h>
#include "view_constants.h"
#include "ViewManager.h"
#include "BallView.h"
#include "GameView.h"
#include "CommandView.h"
//  #include "TVector.h"


RS_BEGIN_NAMESPACE


BallView::BallView(const ViewId id,
		   const UInt hist_len)
  : ViewWithHistory(id),
    parameterView(NULL),
    commandView(NULL),
    agentView(NULL),
    theBall(hist_len),
    lastObservationTime()
{
  //  worldModel->setBallHistoryLength(hist_len);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(gameView, GAME_VIEW_ID, hist_len);
  getView(commandView, COMMAND_VIEW_ID, hist_len);
  getView(agentView, AGENT_VIEW_ID, parameterView, hist_len);
}


BallView::~BallView()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(COMMAND_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}


void BallView::myUpdateAfterInit()
{
  theBall[getIndex(NOW)].setAbsoluteSpeedVector(VectorUCO(0,0));
  theBall[getIndex(NOW)].setPosition(PointUCO(0,0));  
}


// NewCycle is called *before* the values in CSensorInterface
// are erased, so this is the last chance to use them...
void BallView::myNewCycle()
{
  commandView->newCycle(currentUpdateNumber);
  agentView->newCycle(currentUpdateNumber);

  // Start with updating after the immediate actions that have already
  // been executed *last* cycle.
  // If the agent made a turn_neck update direction to the ball
  //if ( commandView->getLastCommandType() == CMD_TurnNeck )
  theBall[getIndex(NOW)].setDirection(theBall[getIndex(NOW)].getDirection() -
				      commandView->getActualTurnNeck(PREV));

  // If the agent made a move update the vector to the ball
  if ( commandView->getMovePosition(PREV).isKnown() ) {
    theBall[getIndex(NOW)].setVector(VectorUCO(agentView->getAgentPosition(PREV),
					       getBallPosition(NOW)).rotate(agentView->getFaceDirection(PREV)));
  }

  // Copy the current ball data and use it as the current data
  // in the next cycle.
  theBall.copy(getIndex(NOW), getIndex(NEXT));
  
  // Update the current observation.
  increaseIndex();

  // Update the ball vector with the movement of the agent
#if 1
  theBall[getIndex(NOW)].setVector(VectorUCO(agentView->getAgentPosition(NOW),
        getBallPosition(PREV)).rotate(-agentView->getFaceDirection(NOW)));
	   //agentView->getAgentObject(NOW).getRelativeVectorTo(theBall[getIndex(NOW)].getPosition()));
#else
  theBall[getIndex(NOW)].setVector(theBall[getIndex(NOW)].getVector() - commandView->getAgentMovement(PREV));
  
  // Update the ball with the actual turn angle of the agent
  theBall[getIndex(NOW)].setDirection(theBall[getIndex(NOW)].getDirection() -
				      commandView->getActualTurn(PREV));
#endif

  // Update the ball position based on the estimation done in the command view
  // Hack by martin
  if (commandView->getBallMovement(PREV).getLength() < 2)
    //theBall[getIndex(NOW)].SetVector(VectorUCO(agentView->getAgentPosition(NOW),
    //						getBallPosition(NOW) +
    //  					commandView->getBallMovement(PREV)));
    theBall[getIndex(NOW)].setVector(theBall[getIndex(NOW)].getVector() +
				     commandView->getBallMovement(PREV));

  // Update the speed of the ball
  theBall[getIndex(NOW)].setAbsoluteSpeedVector(VectorUCO(VectorUnknown(commandView->getBallSpeed(NOW))));

  /*
  if ( getBallSpeed(NOW).IsKnown() ) {
    theBall[getIndex(NOW)].SetSpeed(theBall[getIndex(NOW)].getSpeed()*
				    parameterView->SP_ball_decay);
    if ( theBall[getIndex(NOW)].getAbsoluteSpeedDirection().IsKnown() ) {
      theBall[getIndex(NOW)].SetVector(theBall[getIndex(NOW)].getVector()+
				       theBall[getIndex(NOW)].getAbsoluteSpeedVector());
    }
  }
  */

  PlayMode pm = gameView->getPlayMode(NOW);
  if ( pm == PM_BEFORE_KICK_OFF ||
       pm == PM_KICK_OFF ||
       pm == PM_OUR_KICK_OFF ||
       pm == PM_THEIR_KICK_OFF ) {
    theBall[getIndex(NOW)].setAbsoluteSpeedVector(VectorUCO(0,0));
    //theBall[getIndex(NOW)].setPosition(PointUCO(0,0));
    theBall[getIndex(NOW)].setVector(VectorUCO(agentView->getAgentPosition(NOW), PointUCO(0,0)).
				     rotate(-agentView->getFaceDirection(NOW)));
  }
}


void BallView::myUpdateAfterSee()
{
  agentView->updateAfterSee(currentUpdateNumber);

  if ( sensorInterface->sawBall() ) {

    theBall[getIndex(NOW)].setDirection(sensorInterface->getBallDirection());
    theBall[getIndex(NOW)].setDistance(sensorInterface->getBallDistance());
    theBall[getIndex(NOW)].setDistanceChange(sensorInterface->getBallDistanceChange());
    theBall[getIndex(NOW)].setDirectionChange(sensorInterface->getBallDirectionChange());
    // Update the speed of the ball
    //VectorUnknown ballSpeed(sensorInterface->getBallSpeedDirection(),
    //		    sensorInterface->getBallSpeedAmount());
    /*
    VectorUnknown ballSpeed1 = ballSpeed - agentView->getSpeedVector(NOW);
    VectorUnknown ballSpeed2 = ballSpeed + agentView->getSpeedVector(NOW);
    RS_LOG( LA_BALL_VIEW, "Ballspeed = " << ballSpeed );
    RS_LOG( LA_BALL_VIEW, "BallSpeed - AgentSpeed = " << ballSpeed1 );
    RS_LOG( LA_BALL_VIEW, "BallSpeed + AgentSpeed = " << ballSpeed2 );
    */
    /*
    VectorUnknown ballSpeed3 = ballSpeed.rotate(sensorInterface->getBallDirection());
    std::cerr << "BallSpeed.rotate(ballDir) = " << ballSpeed3 << std::endl;
    VectorUnknown ballSpeed4 = ballSpeed.rotate(-(sensorInterface->getBallDirection()));
    std::cerr << "BallSpeed.rotate(-ballDir) = " << ballSpeed4 << std::endl;
    ballSpeed3 -= agentView->getSpeedVector(NOW);
    std::cerr << "BallSpeed.rotate(ballDir) - AgentSpeed = " << ballSpeed3 << std::endl;
    ballSpeed4 += agentView->getSpeedVector(NOW);
    std::cerr << "BallSpeed.rotate(-ballDir) + AgentSpeed = " << ballSpeed4 << std::endl;
    */
    //theBall[getIndex(NOW)].setAbsoluteSpeedVector(ballSpeed);
    
    // Update the position of the ball
    theBall[getIndex(NOW)].setPosition(agentView->getAgentPosition(NOW) +
				       theBall[getIndex(NOW)].getVector());

    // Warning a temporary hack!!! TO BE REMOVED!!!
    lastObservationTime = gameView->getCurrentTime();

    // Do an estimation of the ball speed, not good!
    estimateBallSpeed();
  } else {
    theBall[getIndex(NOW)].setDistanceChange(FloatUCO());
    theBall[getIndex(NOW)].setDirectionChange(FloatUCO());
    //theBall[getIndex(NOW)].setAbsoluteSpeedVector(VectorUCO());

    // If I can't see the ball, but should see it, I don't know where it is.
    if ( ( abs(getRelativeBallDirection(NOW)) < agentView->getViewAngle(NOW)/2.0 ) ||
	 ( getBallDistance(NOW) < parameterView->SP_feel_distance ) ) {
      theBall[getIndex(NOW)].setPosition(PointUnknown());
      theBall[getIndex(NOW)].setVector(VectorUnknown());
    }
  }

  PlayMode pm = gameView->getPlayMode(NOW);
  if ( pm == PM_BEFORE_KICK_OFF ||
       pm == PM_KICK_OFF ||
       pm == PM_OUR_KICK_OFF ||
       pm == PM_THEIR_KICK_OFF ) {
    theBall[getIndex(NOW)].setAbsoluteSpeedVector(VectorUCO(0,0));
    //theBall[getIndex(NOW)].setPosition(Point(0,0));
    theBall[getIndex(NOW)].setVector(VectorUCO(agentView->getAgentPosition(NOW), PointUCO(0,0)).
				     rotate(-agentView->getFaceDirection(NOW)));
  }

}


void BallView::myUpdateAfterHear()
{
  agentView->updateAfterHear(currentUpdateNumber);
  //markerView->updateAfterHear(currentUpdateNumber);

  PlayMode pm = gameView->getPlayMode(NOW);
  if ( pm == PM_BEFORE_KICK_OFF ||
       pm == PM_KICK_OFF ||
       pm == PM_OUR_KICK_OFF ||
       pm == PM_THEIR_KICK_OFF ) {
    theBall[getIndex(NOW)].setAbsoluteSpeedVector(VectorUCO(0,0));
    //theBall[getIndex(NOW)].setPosition(Point(0,0));
    theBall[getIndex(NOW)].setVector(VectorUCO(agentView->getAgentPosition(NOW), PointUCO(0,0)).
				     rotate(-agentView->getFaceDirection(NOW)));

  }
}


void BallView::myUpdateAfterCommand()
{
  /* Since we currently send all commands last in the cycle, even
     immediate actions, we can do all the updating at the begining
     of the new cycle.
  commandView->UpdateAfterCommand(currentUpdateNumber);
  agentView->UpdateAfterCommand(currentUpdateNumber);

  // If the agent made a turn_neck update direction to the ball
  if ( commandView->getLastCommandType() == CMD_TurnNeck )
    theBall[getIndex(NOW)].setDirection(theBall[getIndex(NOW)].getDirection() -
					commandView->getActualTurnNeck(NOW));

  // If the agent made a move update the vector to the ball
  if ( commandView->getLastCommandType() == CMD_Move ) {
    theBall[getIndex(NOW)].setVector(VectorUCO(agentView->getAgentPosition(NOW),
    getBallPosition(NOW)));
  }
  */
}


void BallView::estimateBallSpeed()
{
  /**
   * Absolute Velocity = Player Velocity + Relative Velocity
   *
   * Relative Velocity.x = DistChg * Unit.x - ( DirChg * pi/180 ) * Dist * Unit.y
   * Relative Velocity.y = DistChg * Unit.y + ( DirChg * pi/180 ) * Dist * Unit.x
   *
   * Unit = Relative Position / Dist
   *
   * Relative Position = Absolute Position - Player Position
   */

   // If I've got dist_chng and dir_chng use them
  if ( getBallObject(NOW).getDistanceChange().isKnown() &&
       getBallObject(NOW).getDirectionChange().isKnown()) {

    PointUnknown relPos = getBallObject(NOW).getPosition() -
      agentView->getAgentPosition(NOW);
    PointUnknown unit = relPos / getBallObject(NOW).getDistance();
    FloatUnknown tmp = getBallObject(NOW).getDirectionChange().getRad() *
      getBallObject(NOW).getDistance();
    FloatUnknown relVelX = getBallObject(NOW).getDistanceChange() * unit.getX() -
      tmp*unit.getY();
    FloatUnknown relVelY = getBallObject(NOW).getDistanceChange() * unit.getY() +
      tmp*unit.getX();
    
    VectorUnknown relVel(PointUnknown(relVelX, relVelY));
    VectorUnknown absVel = relVel;
    absVel.rotate(agentView->getFaceDirection(NOW));

    // We need to compensate for agent decay.
    VectorUnknown agV = agentView->getSpeedVector(NOW);
    absVel += agV;

    theBall[getIndex(NOW)].setAbsoluteSpeedVector(absVel);

//      TVector vr = TVector( absVel );
//      TVector va = TVector( agV );
    //      cout << "(" << getBallObject(NOW).getDistanceChange().getValue() << ", "
//  	 << getBallObject(NOW).getDirectionChange().getDeg() << ")" << endl;
//      cout << "(" << vr.getX() << "+i*" << vr.getY() << ") " << flush;
//      cout << "(" << va.getX() << "+i*" << va.getY() << ") " << flush;

    /*
    FloatUnknown Ex = cos((getBallObject(NOW).getDirection() / 180.0) * PI);
    FloatUnknown Ey = sin((getBallObject(NOW).getDirection() / 180.0) * PI);
    FloatUnknown Wx = (getBallObject(NOW).getDistanceChange() * Ex)
      + (((getBallObject(NOW).getDirectionChange().getDeg() * PI) / 180.0)
	 * getBallObject(NOW).getDistance() * Ey);
    FloatUnknown Wy = (getBallObject(NOW).getDistanceChange() * Ey)
      + (((getBallObject(NOW).getDirectionChange().getDeg() * PI) / 180.0)
	 * getBallObject(NOW).getDistance() * Ex);
    VectorUnknown rel_vel = VectorUnknown(PointUnknown(Wx, Wy));
    
    rel_vel.rotate(agentView->getBodyDirection(NOW));
    rel_vel += agentView->getSpeedVector(NOW);
    rel_vel.rotate(-agentView->getBodyDirection(NOW));
    theBall[getIndex(NOW)].setAbsoluteSpeedVector(rel_vel);
    */
    
    //rel_vel.Rotate(-body_direction);
    //std::cout << "Estimated ball speed: " << ballRelSpeed
    //  << std::endl
    
    /* from CMU doesn't work
       // Switched the order...
       Vector rel_vel(Point(getBallObject(NOW).getDistChanged(NOW),
       getBallObject(NOW).getDistance(NOW)*
       Tan(getBallObject(NOW).getDirChanged(NOW))));
       rel_vel.Rotate(getBallObject(NOW).getDirection(NOW));
       Vector abs_vel = k->theAgent().getAbsoluteSpeedVector(NOW) + rel_vel.Rotate(body_direction);
       */
    /*
      Vector abs_vel = Vector(getBallObject(NOW).getDirChanged(NOW),
      getBallObject(NOW).getDistChanged(NOW))
      - k->theAgent().getAbsoluteSpeedVector(NOW);
      getBallObject(NOW).setAbsoluteSpeedVector(abs_vel, NOW);
      */
    //VectorUnknown abs_vel = rel_vel.Rotate(-agentView->getBodyDirection(NOW));
    //theBall[getIndex(NOW)].setAbsoluteSpeedVector(abs_vel);
//     if ( showInfo ) {
//       std::cout << "Calculated the ball speed to: " << abs_vel << std::endl;
//       std::cout << "rel_vel: " << rel_vel << " RelVel: "
// 		<< getBallObject(NOW).getRelativeSpeedDirection(NOW)
// 		<< std::endl;
//       std::cout << "my speed: " << k->theAgent().getAbsoluteSpeedVector(NOW)
// 		<< " body_dir: " << body_direction
// 		<< " ball_dir: " << getBallObject(NOW).getDirection(NOW)
// 		<< std::endl;
//       logf->LogText("Updated the ball speed.");
//     }
  } else {
    theBall[getIndex(NOW)].setAbsoluteSpeedVector(VectorUnknown());	
  }

  // Estimate the speed of the ball
  // REALLY bad!
  
  //std::cout << "Ball rel_vel: " << rel_vel
  //	<< " abs_vel: " << abs_vel << std::endl;
  //getBallObject(NOW).setAbsoluteSpeedVector(Vector(getBallObject(NOW).getPosition(PREV), pos), NOW);
  // int_type t = getBallObject(NOW).getObsTime(-HIST + 1) - k->theGame().getCurrentTime();
}


bool BallView::isBallCatchable(const Int hist) const
{
  // First rotate the ball vector -bodydir degrees.
  AngleDegUnknown bodyDir = agentView->getAgentObject(NOW).getBodyDirection();
  PointUCO ballPt = PointUCO(getBallObject(hist).getVector().rotate(-bodyDir));
  // Then check whether the x and y distances are less than
  // catchable area width and catchable area length.
  return ( abs(ballPt.getX()) < parameterView->SP_catchable_area_w &&
	   ballPt.getY() < parameterView->SP_catchable_area_l &&
	   ballPt.getY() > 0 );
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "BallView.icc"
#endif
