/* -*- Mode: C++ -*-
 * $Id: WorldStateView.h,v 1.1 2001/09/18 20:41:50 andbo928 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WorldStateView.cc
 * Unitname: Framework/view
 * $Revision: 1.1 $
 * Created by: Mikael Brnnstrm, Andreas Sigfridsson, Andreas Bckert
 * Last modified by $Author: andbo928 $ $Date: 2001/09/18 20:41:50 $
 *
 *
 * HISTORY
 *
 * $Log: WorldStateView.h,v $
 * Revision 1.1  2001/09/18 20:41:50  andbo928
 * Imported some sources from FuzzyFoo.
 *
 * Revision 1.15  2001/07/06 16:28:44  mikbr212
 * Fixed nextSeeCycle-phenomena
 *
 * Revision 1.14  2001/07/06 10:00:39  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.13  2001/07/02 14:44:59  andbo928
 * Cosmetic changes.
 *
 * Revision 1.12  2001/07/01 12:40:35  andbo928
 * Observations now merge onNewCycle
 *
 * Revision 1.11  2001/06/19 14:01:00  mikbr212
 * Fixed circular includes stuff. Somebody should check that I did it right.
 *
 * Revision 1.10  2001/04/24 20:43:17  mikbr212
 * added possession
 *
 * Revision 1.9  2001/04/24 09:54:47  sigge
 * Moved communication policy to communication view. Communication view no longer erases messages not picked up from the incoming queue in order to be useful to skills.
 *
 * Revision 1.8  2001/04/22 16:00:56  mikbr212
 * adjusted agent-speed direction
 *
 * Revision 1.7  2001/04/16 14:11:19  mikbr212
 * added magic ball stuff
 *
 * Revision 1.6  2001/04/13 16:27:38  sigge
 * Added communication
 *
 * Revision 1.5  2001/04/12 13:44:23  mikbr212
 * added includes
 *
 * Revision 1.4  2001/04/12 11:22:16  mikbr212
 * added some stuff
 *
 * Revision 1.3  2001/04/11 18:31:38  mikbr212
 * added getWorldState
 *
 * Revision 1.2  2001/04/11 18:19:25  mikbr212
 * Implemented the basics
 *
 * Revision 1.1  2001/04/11 09:10:25  mikbr212
 * Created view
 *
 *
 */

/**
 * @file WorldStateView.h
 * Implementation file for the class WorldStateView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: andbo928 $
 * $Date: 2001/09/18 20:41:50 $
 */



#ifndef WORLDSTATEVIEW_H
#define WORLDSTATEVIEW_H

#include "View.h"
#include "view_constants.h"

#include "TVector.h"

#include "WorldState.h"
#include "PlayerState.h"
#include "BallState.h"

#include "MoveCommand.h"
#include "TurnNeckCommand.h"
#include "TurnCommand.h"
#include "DashCommand.h"
#include "KickCommand.h"
#include "CatchCommand.h"

RS_BEGIN_NAMESPACE

// class CommunicationView;
class ParameterView;
class MarkerView;
class GameView;

/**
 * @class WorldStateView WorldStateView.h
 * This view is used to keep track of the ball.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: andbo928 $
 * $Date: 2001/09/18 20:41:50 $
 */
class WorldStateView : public View {
public:
  WorldStateView(const ViewId id);

  /** Destroys the world state view. */
  virtual ~WorldStateView();

  const WorldState& getWorldState() const;

  int getLastSeeCycle() const;
  int getNextSeeCycle() const;
  int getNextNextSeeCycle() const;
  
protected:
  WorldState ws; // The world state
  WorldState obs; // The observation
  WorldState lastWs; // The world state

  bool newObservations;

  Time lastSeeTime;
  int lastSeeCycle;
  int nextSeeCycle;
  int nextNextSeeCycle;
  float lastDashBodyDir;
  ServerCommand* lastCommand;
  TurnNeckCommand* lastTurnNeckCommand;

  std::vector<PlayerState> obsTeammates;
  std::vector<PlayerState> obsOpponents;

  ParameterView* parameterView;
  MarkerView* markerView;
  GameView* gameView;
  //  CommunicationView* communicationView;

  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  void updateAgentAfterSee();
  void updateBallAfterSee();
  void updatePlayersAfterSee();
  bool magicBallSpeedComputation(BallState& ball, double dist);

  void processPlayer(const ServerPlayerObject& plr);
  TVector computePosition(const float relDist, const float relDir,
			  const AgentState& agent);
  TVector computeSpeed(const float distChange, const float dirChange, 
		       const float relDist, const float relDir,
		       const AgentState& agent);
private:
  // Hide copy-constructor and assignment.
  WorldStateView(const WorldStateView&);
  WorldStateView& operator= (const WorldStateView&);  
};

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "WorldStateView.icc"
#endif


#endif // !WORLDSTATEVIEW_H

