/* -*- Mode: C++ -*-
 * $Id: AngleDegreesUC.h,v 1.6 2001/09/01 23:53:08 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AngleDegreesUC.h
 * Unitname: Library
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-08-31
 * Last modified by $Author: frehe $ $Date: 2001/09/01 23:53:08 $
 *
 *
 * HISTORY
 *
 * $Log: AngleDegreesUC.h,v $
 * Revision 1.6  2001/09/01 23:53:08  frehe
 * Changed the implementation of getValue and getAngle.
 *
 * Revision 1.5  2001/08/29 21:08:27  frehe
 * Replaces class T with typename T where T can be a built in type.
 *
 * Revision 1.4  2000/09/18 15:20:03  frehe
 * Due to a problem with namespaces the name for atan2, atan, acos and asin is changed to ATan2, ATan, ASin and ACos.
 *
 * Revision 1.3  2000/09/03 14:23:18  frehe
 * Made minor changes to improve the documentation.
 *
 * Revision 1.2  2000/09/01 03:23:50  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:36  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AngleDegreesUC.h
 * Include file for the template AngleDegreesUC.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:53:08 $
 */


#ifndef ANGLEDEGUC_H
#define ANGLEDEGUC_H


#include "AngleDegreesUnknown.h"
#include "WithConfidence.h"


RS_BEGIN_NAMESPACE


/**
 * @class AngleDegreesUC AngleDegreesUC.h
 * Template class for representing angles in degrees
 * that can be unknown and have a confidence factor.
 * The template argument only defines what type the
 * angles should be represented by.
 * The only demands on the template argument T is that the trigonometric
 * and arithmetic functions are defined on it.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:53:08 $
 */
template<typename T>
class AngleDegreesUC : public AngleDegreesUnknown<T> {
public:
  AngleDegreesUC();
  AngleDegreesUC(const T& v,
		 const Confidence& conf = Confidence(),
		 const bool print_conf = true);
  AngleDegreesUC(const AngleDegrees<T>& ang,
		 const Confidence& conf = Confidence(),
		 const bool print_conf = true);
  AngleDegreesUC(const AngleRadians<T>& ang,
		 const Confidence& conf = Confidence(),
		 const bool print_conf = true);
  AngleDegreesUC(const AngleDegreesUnknown<T>& au,
		 const Confidence& conf = Confidence(),
		 const bool print_conf = true);
  AngleDegreesUC(const WithUnknown<T>& wu,
		 const Confidence& conf = Confidence(),
		 const bool print_conf = true);
  AngleDegreesUC(const AngleDegreesUC<T>& wuc);

  //AngleDegreesUnknown<T> getAngle() const;

  Confidence getConfidence() const;
  void setConfidence(const Confidence& conf);

  bool printConfidence() const;
  void setPrintConfidence(const bool print_confidence);
  
  AngleDegreesUC<T>& operator= (const AngleDegreesUC<T>& op);

  AngleDegreesUC<T>& operator+= (const AngleDegreesUC<T>& op);
  AngleDegreesUC<T>& operator-= (const AngleDegreesUC<T>& op);
  AngleDegreesUC<T>& operator*= (const AngleDegreesUC<T>& op);
  AngleDegreesUC<T>& operator/= (const AngleDegreesUC<T>& op);

  AngleDegreesUC<T> operator- () const;
  AngleDegreesUC<T> operator+ (const AngleDegreesUC<T>& op) const;
  AngleDegreesUC<T> operator- (const AngleDegreesUC<T>& op) const;
  AngleDegreesUC<T> operator* (const AngleDegreesUC<T>& op) const;
  AngleDegreesUC<T> operator/ (const AngleDegreesUC<T>& op) const;
  
protected:
  Confidence confidence;
  bool doPrintConfidence;
};

template<typename T>
AngleDegreesUC<T>::AngleDegreesUC()
  : AngleDegreesUnknown<T>(),
    confidence(),
    doPrintConfidence(true)
{
}

template<typename T>
AngleDegreesUC<T>::AngleDegreesUC(const T& v, const Confidence& conf, const bool print_conf)
  : AngleDegreesUnknown<T>(v),
    confidence(conf),
    doPrintConfidence(print_conf)
{
}

template<typename T>
AngleDegreesUC<T>::AngleDegreesUC(const AngleDegrees<T>& ang,
			  const Confidence& conf,
			  const bool print_conf)
  : AngleDegreesUnknown<T>(ang),
    confidence(conf),
    doPrintConfidence(print_conf)
{
}

template<typename T>
AngleDegreesUC<T>::AngleDegreesUC(const AngleRadians<T>& ang,
			  const Confidence& conf,
			  const bool print_conf)
  : AngleDegreesUnknown<T>(ang),
    confidence(conf),
    doPrintConfidence(print_conf)
{
}

template<typename T>
AngleDegreesUC<T>::AngleDegreesUC(const AngleDegreesUnknown<T>& wu,
			  const Confidence& conf,
			  const bool print_conf)
  : AngleDegreesUnknown<T>(wu),
    confidence(conf),
    doPrintConfidence(print_conf)
{
}

template<typename T>
AngleDegreesUC<T>::AngleDegreesUC(const WithUnknown<T>& wu,
			  const Confidence& conf,
			  const bool print_conf)
  : AngleDegreesUnknown<T>(wu),
    confidence(conf),
    doPrintConfidence(print_conf)
{
}

template<typename T>
AngleDegreesUC<T>::AngleDegreesUC(const AngleDegreesUC<T>& wuc)
  : AngleDegreesUnknown<T>(wuc),
    confidence(wuc.confidence),
    doPrintConfidence(wuc.doPrintConfidence)
{
}

/*
template<typename T>
inline AngleDegreesUnknown<T> AngleDegreesUC<T>::getAngle() const
{
  return AngleDegreesUnknown<T>(*this);
}
*/

template<typename T>
inline Confidence AngleDegreesUC<T>::getConfidence() const
{
  return confidence;
}

template<typename T>
inline void AngleDegreesUC<T>::setConfidence(const Confidence& conf)
{
  confidence = conf;
}

template<typename T>
inline bool AngleDegreesUC<T>::printConfidence() const
{
  return doPrintConfidence;
}

template<typename T>
inline void AngleDegreesUC<T>::setPrintConfidence(const bool print_conf)
{
  doPrintConfidence = print_conf;
}

template<typename T>
AngleDegreesUC<T>& AngleDegreesUC<T>::operator= (const AngleDegreesUC<T>& op)
{
  if ( &op != this ) {
    AngleDegreesUnknown<T>::operator=(op);
    confidence = op.confidence;
    doPrintConfidence = op.doPrintConfidence;
  }

  return *this;
}

template<typename T>
AngleDegreesUC<T>& AngleDegreesUC<T>::operator+= (const AngleDegreesUC<T>& op)
{
  AngleDegreesUnknown<T>::operator+=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  confidence = min(confidence, op.confidence);
#else
  confidence = max(confidence, op.confidence);
#endif
  return *this;
}

template<typename T>
AngleDegreesUC<T>& AngleDegreesUC<T>::operator-= (const AngleDegreesUC<T>& op)
{
  AngleDegreesUnknown<T>::operator-=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  confidence = min(confidence, op.confidence);
#else
  confidence = max(confidence, op.confidence);
#endif
  return *this;
}

template<typename T>
AngleDegreesUC<T>& AngleDegreesUC<T>::operator*= (const AngleDegreesUC<T>& op)
{
  AngleDegreesUnknown<T>::operator*=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  confidence = min(confidence, op.confidence);
#else
  confidence = max(confidence, op.confidence);
#endif
  return *this;
}

template<typename T>
AngleDegreesUC<T>& AngleDegreesUC<T>::operator/= (const AngleDegreesUC<T>& op)
{
  AngleDegreesUnknown<T>::operator/=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  confidence = min(confidence, op.confidence);
#else
  confidence = max(confidence, op.confidence);
#endif
  return *this;
}

template<typename T>
inline AngleDegreesUC<T> AngleDegreesUC<T>::operator- () const
{
  return AngleDegreesUC<T>(AngleDegreesUnknown<T>::operator-(),
			   confidence, doPrintConfidence);
}

template<typename T>
inline AngleDegreesUC<T> AngleDegreesUC<T>::operator+ (const AngleDegreesUC<T>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(AngleDegreesUnknown<T>::operator+(op),
		       min(confidence, op.confidence),
		       doPrintConfidence);
#else
  return AngleDegreesUC<T>(AngleDegreesUnknown<T>::operator+(op),
		       max(confidence, op.confidence),
		       doPrintConfidence);
#endif
}

template<typename T>
inline AngleDegreesUC<T> AngleDegreesUC<T>::operator- (const AngleDegreesUC<T>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(AngleDegreesUnknown<T>::operator-(op),
		       min(confidence, op.confidence),
		       doPrintConfidence);
#else
  return AngleDegreesUC<T>(AngleDegreesUnknown<T>::operator-(op),
		       max(confidence, op.confidence),
		       doPrintConfidence);
#endif
}

template<typename T>
inline AngleDegreesUC<T> AngleDegreesUC<T>::operator* (const AngleDegreesUC<T>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(AngleDegreesUnknown<T>::operator*(op),
		       min(confidence, op.confidence),
		       doPrintConfidence);
#else
  return AngleDegreesUC<T>(AngleDegreesUnknown<T>::operator*(op),
		       max(confidence, op.confidence),
		       doPrintConfidence);
#endif
}

template<typename T>
inline AngleDegreesUC<T> AngleDegreesUC<T>::operator/ (const AngleDegreesUC<T>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(AngleDegreesUnknown<T>::operator/(op),
		       min(confidence, op.confidence),
		       doPrintConfidence);
#else
  return AngleDegreesUC<T>(AngleDegreesUnknown<T>::operator/(op),
		       max(confidence, op.confidence),
		       doPrintConfidence);
#endif
}

template<typename T>
inline AngleDegreesUC<T> operator+ (const T& op1, const AngleDegreesUC<T>& op2)
{
  return op2 + op1;
}

template<typename T>
inline AngleDegreesUC<T> operator- (const T& op1, const AngleDegreesUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(op1 - AngleDegreesUnknown<T>(op2),
		   min(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return AngleDegreesUC<T>(op1 - AngleDegreesUnknown<T>(op2),
		   max(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
inline AngleDegreesUC<T> operator* (const T& op1, const AngleDegreesUC<T>& op2)
{
  return op2 * op1;
}

template<typename T>
inline AngleDegreesUC<T> operator/ (const T& op1, const AngleDegreesUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(op1 / AngleDegreesUnknown<T>(op2),
		   min(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return AngleDegreesUC<T>(op1 / AngleDegreesUnknown<T>(op2),
		   max(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
inline AngleDegreesUC<T> operator+ (const AngleDegrees<T>& op1, const AngleDegreesUC<T>& op2)
{
  return op2 + op1;
}

template<typename T>
inline AngleDegreesUC<T> operator- (const AngleDegrees<T>& op1, const AngleDegreesUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(op1 - AngleDegreesUnknown<T>(op2),
		       min(Confidence(), op2.getConfidence()),
		       op2.printConfidence());
#else
  return AngleDegreesUC<T>(op1 - AngleDegreesUnknown<T>(op2),
		       max(Confidence(), op2.getConfidence()),
		       op2.printConfidence());
#endif
}

template<typename T>
inline AngleDegreesUC<T> operator* (const AngleDegrees<T>& op1, const AngleDegreesUC<T>& op2)
{
  return op2 * op1;
}

template<typename T>
inline AngleDegreesUC<T> operator/ (const AngleDegrees<T>& op1, const AngleDegreesUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(op1 / AngleDegreesUnknown<T>(op2),
		   min(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return AngleDegreesUC<T>(op1 / AngleDegreesUnknown<T>(op2),
		   max(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
inline AngleDegreesUC<T> operator+ (const AngleDegreesUnknown<T>& op1, const AngleDegreesUC<T>& op2)
{
  return op2 + op1;
}

template<typename T>
inline AngleDegreesUC<T> operator- (const AngleDegreesUnknown<T>& op1, const AngleDegreesUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(op1 - AngleDegreesUnknown<T>(op2),
		   min(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return AngleDegreesUC<T>(op1 - AngleDegreesUnknown<T>(op2),
		   max(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
inline AngleDegreesUC<T> operator* (const AngleDegreesUnknown<T>& op1,
				const AngleDegreesUC<T>& op2)
{
  return op2 * op1;
}

template<typename T>
inline AngleDegreesUC<T> operator/ (const AngleDegreesUnknown<T>& op1,
				const AngleDegreesUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(op1 / AngleDegreesUnknown<T>(op2),
		   min(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return AngleDegreesUC<T>(op1 / AngleDegreesUnknown<T>(op2),
		   max(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
inline AngleDegreesUC<T> operator+ (const WithConfidence<T>& op1,
				const AngleDegreesUC<T>& op2)
{
  return op2 + op1;
}

template<typename T>
inline AngleDegreesUC<T> operator- (const WithConfidence<T>& op1,
				const AngleDegreesUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(op1 - AngleDegreesUnknown<T>(op2),
		   min(op1.getConfidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return AngleDegreesUC<T>(op1 - AngleDegreesUnknown<T>(op2),
		   max(op.getConfidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
inline AngleDegreesUC<T> operator* (const WithConfidence<T>& op1,
				const AngleDegreesUC<T>& op2)
{
  return op2 * op1;
}

template<typename T>
inline AngleDegreesUC<T> operator/ (const WithConfidence<T>& op1,
				const AngleDegreesUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUC<T>(op1 / AngleDegreesUnknown<T>(op2),
		       min(getConfidence(), op2.getConfidence()),
		       op2.printConfidence());
#else
  return AngleDegreesUC<T>(op1 / AngleDegreesUnknown<T>(op2),
		       max(getConfidence(), op2.getConfidence()),
		       op2.printConfidence());
#endif
}

template<typename T>
std::ostream& operator<<(std::ostream& os, const AngleDegreesUC<T>& obj)
{
  if (obj.printConfidence())
    return os << AngleDegreesUnknown<T>(obj) << " " << obj.getConfidence();
  else
    return os << AngleDegreesUnknown<T>(obj);
}


#ifdef RS_TRIG_IN_DEG


template<typename T>
inline AngleDegreesUC<T> ASin(const WithUC<T>& op)
{
  //  if ( op.isUnknown() )
  //    return AngleDegreesUC<T>();
  //  else
    return AngleDegreesUC<T>(ASin(WithUnknown<T>(op)));
}


template<typename T>
inline AngleDegreesUC<T> ACos(const WithUC<T>& op)
{
  //  if ( op.isUnknown() )
  //    return AngleDegreesUC<T>();
  //  else
    return AngleDegreesUC<T>(ACos(WithUnknown<T>(op)));
}


template<typename T>
inline AngleDegreesUC<T> ATan(const WithUC<T>& op)
{
  //  if ( op.isUnknown() )
  //    return AngleDegreesUC<T>();
  //  else
    return AngleDegreesUC<T>(ATan(WithUnknown<T>(op)));
}


template<typename T>
inline AngleDegreesUC<T> ATan2(const WithUC<T>& op1, const WithUC<T>& op2)
{
  //  if ( op1.isUnknown() ||
  //       op2.isUnknown() )
  //    return AngleDegreesUC<T>();
  //  else
    return AngleDegreesUC<T>(ATan2(WithUnknown<T>(op1), WithUnknown<T>(op2)));
}


#endif // RS_TRIG_IN_DEG


RS_END_NAMESPACE


#endif // !ANGLEDEGREESUC_H
