/* -*- Mode: C++ -*-
 * $Id: AngleDegreesUCO.h,v 1.6 2001/09/01 23:53:08 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AngleDegreesUCO.h
 * Unitname: Library
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-08-31
 * Last modified by $Author: frehe $ $Date: 2001/09/01 23:53:08 $
 *
 *
 * HISTORY
 *
 * $Log: AngleDegreesUCO.h,v $
 * Revision 1.6  2001/09/01 23:53:08  frehe
 * Changed the implementation of getValue and getAngle.
 *
 * Revision 1.5  2001/08/29 21:08:27  frehe
 * Replaces class T with typename T where T can be a built in type.
 *
 * Revision 1.4  2000/09/18 15:20:04  frehe
 * Due to a problem with namespaces the name for atan2, atan, acos and asin is changed to ATan2, ATan, ASin and ACos.
 *
 * Revision 1.3  2000/09/03 14:23:18  frehe
 * Made minor changes to improve the documentation.
 *
 * Revision 1.2  2000/09/01 03:23:51  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:36  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AngleDegreesUCO.h
 * Include file for the template AngleDegreesUCO.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:53:08 $
 */


#ifndef ANGLEDEGREESUCO_H
#define ANGLEDEGREESUCO_H


#include "WithObservationTime.h"
#include "AngleDegreesUC.h"


RS_BEGIN_NAMESPACE


/**
 * @class AngleDegreesUCO AngleDegreesUCO.h
 * Template class for representing angles in degrees
 * that can be unknown, have a confidence factor and an
 * observation time.
 * The template argument only defines what type the
 * angles should be represented by.
 * The only demands on the template argument T is that the trigonometric
 * and arithmetic functions are defined on it.
 * The demands on Time are...
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:53:08 $
 */
template<typename T, class Time> class AngleDegreesUCO : public AngleDegreesUC<T> {
public:
  AngleDegreesUCO();
  AngleDegreesUCO(const T& v,
		  const Time& t = Time(),
		  const Confidence& conf = Confidence(),
		  const bool print_obs_time = true,
		  const bool print_conf = true);
  AngleDegreesUCO(const AngleDegrees<T>& wu,
		  const Time& t = Time(),
		  const Confidence& conf = Confidence(),
		  const bool print_obs_time = true,
		  const bool print_conf = true);
  AngleDegreesUCO(const AngleRadians<T>& wu,
		  const Time& t = Time(),
		  const Confidence& conf = Confidence(),
		  const bool print_obs_time = true,
		  const bool print_conf = true);
  AngleDegreesUCO(const AngleDegreesUnknown<T>& wu,
		  const Time& t = Time(),
		  const Confidence& conf = Confidence(),
		  const bool print_obs_time = true,
		  const bool print_conf = true);
  AngleDegreesUCO(const WithUnknown<T>& wu,
		  const Time& t = Time(),
		  const Confidence& conf = Confidence(),
		  const bool print_obs_time = true,
		  const bool print_conf = true);
  AngleDegreesUCO(const AngleDegreesUC<T>& wuc,
		  const Time& t = Time(),
		  const bool print_obs_time = true);
  AngleDegreesUCO(const AngleDegreesUCO<T, Time>&  wuco);

  //AngleDegreesUC<T> getAngle() const;
  Time getObservationTime() const;
  void setObservationTime(const Time& obs_time);
  UInt16U getCyclesSince(const Time& cur_time) const;

  bool printObservationTime() const;
  void setPrintObservationTime(const bool print_obs_time);
  
  AngleDegreesUCO<T, Time>& operator= (const AngleDegreesUCO<T, Time>& op);

  AngleDegreesUCO<T, Time>& operator+= (const AngleDegreesUCO<T, Time>& op);
  AngleDegreesUCO<T, Time>& operator-= (const AngleDegreesUCO<T, Time>& op);
  AngleDegreesUCO<T, Time>& operator*= (const AngleDegreesUCO<T, Time>& op);
  AngleDegreesUCO<T, Time>& operator/= (const AngleDegreesUCO<T, Time>& op);

  AngleDegreesUCO<T, Time> operator- () const;
  AngleDegreesUCO<T, Time> operator+ (const AngleDegreesUCO<T, Time>& op) const;
  AngleDegreesUCO<T, Time> operator- (const AngleDegreesUCO<T, Time>& op) const;
  AngleDegreesUCO<T, Time> operator* (const AngleDegreesUCO<T, Time>& op) const;
  AngleDegreesUCO<T, Time> operator/ (const AngleDegreesUCO<T, Time>& op) const;

private:
  Time observationTime;
  bool doPrintObservationTime;
};

template<typename T, class Time>
AngleDegreesUCO<T, Time>::AngleDegreesUCO()
  : AngleDegreesUC<T>(),
    observationTime(),
    doPrintObservationTime(true)
{
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>::AngleDegreesUCO(const T& v,
				  const Time& t,
				  const Confidence& conf,
				  const bool print_obs_time,
				  const bool print_conf)
  : AngleDegreesUC<T>(v, conf, print_conf),
    observationTime(t),
    doPrintObservationTime(print_obs_time)
{
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>::AngleDegreesUCO(const AngleDegrees<T>& wu,
				  const Time& t,
				  const Confidence& conf,
				  const bool print_obs_time,
				  const bool print_conf)
  : AngleDegreesUC<T>(wu, conf, print_conf),
    observationTime(t),
    doPrintObservationTime(print_obs_time)
{
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>::AngleDegreesUCO(const AngleRadians<T>& wu,
				  const Time& t,
				  const Confidence& conf,
				  const bool print_obs_time,
				  const bool print_conf)
  : AngleDegreesUC<T>(wu, conf, print_conf),
    observationTime(t),
    doPrintObservationTime(print_obs_time)
{
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>::AngleDegreesUCO(const WithUnknown<T>& wu,
				  const Time& t,
				  const Confidence& conf,
				  const bool print_obs_time,
				  const bool print_conf)
  : AngleDegreesUC<T>(wu, conf, print_conf),
    observationTime(t),
    doPrintObservationTime(print_obs_time)
{
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>::AngleDegreesUCO(const AngleDegreesUnknown<T>& wu,
				  const Time& t,
				  const Confidence& conf,
				  const bool print_obs_time,
				  const bool print_conf)
  : AngleDegreesUC<T>(wu, conf, print_conf),
    observationTime(t),
    doPrintObservationTime(print_obs_time)
{
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>::AngleDegreesUCO(const AngleDegreesUC<T>& wuc,
				  const Time& t,
				  const bool print_obs_time)
  : AngleDegreesUC<T>(wuc),
    observationTime(t),
    doPrintObservationTime(print_obs_time)
{
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>::AngleDegreesUCO(const AngleDegreesUCO<T, Time>& wuco)
  : AngleDegreesUC<T>(wuco),
    observationTime(wuco.observationTime),
    doPrintObservationTime(wuco.doPrintObservationTime)
{
}

/*
template <typename T, class Time>
inline AngleDegreesUC<T> AngleDegreesUCO<T, Time>::getAngle() const
{
  return AngleDegreesUC<T>(*this);
}
*/

template <typename T, class Time> 
inline Time AngleDegreesUCO<T, Time>::getObservationTime() const
{
  return observationTime;
}

template <typename T, class Time> 
inline void AngleDegreesUCO<T, Time>::setObservationTime(const Time& obs_time)
{
  observationTime = obs_time;
}

template <typename T, class Time> 
inline UInt16U AngleDegreesUCO<T, Time>::getCyclesSince(const Time& cur_time) const
{
  return observationTime - cur_time;
}

template <typename T, class Time>
inline bool AngleDegreesUCO<T, Time>::printObservationTime() const
{
  return doPrintObservationTime;
}

template <typename T, class Time>
inline void AngleDegreesUCO<T, Time>::setPrintObservationTime(const bool print_obs_time)
{
  doPrintObservationTime = print_obs_time;
}

template <typename T, class Time>
AngleDegreesUCO<T, Time>& AngleDegreesUCO<T, Time>::operator= (const AngleDegreesUCO<T, Time>& op)
{
  if ( &op != this ) {
    AngleDegreesUC<T>::operator=(op);
    observationTime = op.observationTime;
    doPrintObservationTime = op.doPrintObservationTime;
  }

  return *this;
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>& AngleDegreesUCO<T, Time>::operator+= (const AngleDegreesUCO<T, Time>& op)
{
  AngleDegreesUC<T>::operator+=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  observationTime = min(observationTime, op.observationTime);
#else
  observationTime = max(observationTime, op.observationTime);
#endif
  return *this;
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>& AngleDegreesUCO<T, Time>::operator-= (const AngleDegreesUCO<T, Time>& op)
{
  AngleDegreesUC<T>::operator-=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  observationTime = min(observationTime, op.observationTime);
#else
  observationTime = max(observationTime, op.observationTime);
#endif
  return *this;
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>& AngleDegreesUCO<T, Time>::operator*= (const AngleDegreesUCO<T, Time>& op)
{
  AngleDegreesUC<T>::operator*=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  observationTime = min(observationTime, op.observationTime);
#else
  observationTime = max(observationTime, op.observationTime);
#endif
  return *this;
}

template<typename T, class Time>
AngleDegreesUCO<T, Time>& AngleDegreesUCO<T, Time>::operator/= (const AngleDegreesUCO<T, Time>& op)
{
  AngleDegreesUC<T>::operator/=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  observationTime = min(observationTime, op.observationTime);
#else
  observationTime = max(observationTime, op.observationTime);
#endif
  return *this;
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> AngleDegreesUCO<T, Time>::operator- () const
{
  return AngleDegreesUCO<T, Time>(AngleDegreesUC<T>::operator-(),
			  observationTime,
			  doPrintObservationTime);
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> AngleDegreesUCO<T, Time>::operator+ (const AngleDegreesUCO<T, Time>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(AngleDegreesUC<T>::operator+(op),
			  min(observationTime, op.observationTime),
			  doPrintObservationTime);
#else
  return AngleDegreesUCO<T, Time>(AngleDegreesUC<T>::operator+(op),
			  max(observationTime, op.observationTime),
			  doPrintObservationTime);
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> AngleDegreesUCO<T, Time>::operator- (const AngleDegreesUCO<T, Time>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(AngleDegreesUC<T>::operator-(op),
			  min(observationTime, op.observationTime),
			  doPrintObservationTime);
#else
  return AngleDegreesUCO<T, Time>(AngleDegreesUC<T>::operator-(op),
			  max(observationTime, op.observationTime),
			  
			  doPrintObservationTime);
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> AngleDegreesUCO<T, Time>::operator* (const AngleDegreesUCO<T, Time>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(AngleDegreesUC<T>::operator*(op),
			  min(observationTime, op.observationTime),
			  doPrintObservationTime);
#else
  return AngleDegreesUCO<T, Time>(AngleDegreesUC<T>::operator*(op),
			  max(observationTime, op.observationTime),

			  doPrintObservationTime);
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> AngleDegreesUCO<T, Time>::operator/ (const AngleDegreesUCO<T, Time>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(AngleDegreesUC<T>::operator/(op),
			  min(observationTime, op.observationTime),
			  doPrintObservationTime);
#else
  return AngleDegreesUCO<T, Time>(AngleDegreesUC<T>::operator/(op),
			  max(observationTime, op.observationTime),

			  doPrintObservationTime);
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator+ (const T& op1, const AngleDegreesUCO<T, Time>& op2)
{
  return op2 + op1;
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator- (const T& op1, const AngleDegreesUCO<T, Time>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(op1 - AngleDegreesUC<T>(op2),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return AngleDegreesUCO<T, Time>(op1 - AngleDegreesUC<T>(op2),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator* (const T& op1, const AngleDegreesUCO<T, Time>& op2)
{
  return op2 * op1;
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator/ (const T& op1, const AngleDegreesUCO<T, Time>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(op1 / AngleDegreesUC<T>(op2),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return AngleDegreesUCO<T, Time>(op1 / AngleDegreesUC<T>(op2),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator+ (const AngleDegrees<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
  return op2 + op1;
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator- (const AngleDegrees<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(op1 - AngleDegreesUC<T>(op2),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return AngleDegreesUCO<T, Time>(op1 - AngleDegreesUC<T>(op2),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator* (const AngleDegrees<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
  return op2 * op1;
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator/ (const AngleDegrees<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(op1 / AngleDegreesUC<T>(op2),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return AngleDegreesUCO<T, Time>(op1 / AngleDegreesUC<T>(op2),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator+ (const AngleDegreesUC<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
  return op2 + op1;
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator- (const AngleDegreesUC<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(op1 - AngleDegreesUC<T>(op2),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return AngleDegreesUCO<T, Time>(op1 - AngleDegreesUC<T>(op2),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator* (const AngleDegreesUC<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
  return op2 * op1;
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator/ (const AngleDegreesUC<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(op1 / AngleDegreesUC<T>(op2),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return AngleDegreesUCO<T, Time>(op1 / AngleDegreesUC<T>(op2),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator+ (const WithUnknown<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
  return op2 + op1;
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator- (const WithUnknown<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(op1 - AngleDegreesUC<T>(op2),
			      min(Time(), op2.getObservationTime()),
			      op2.printObservationTime());
#else
  return AngleDegreesUCO<T, Time>(op1 - AngleDegreesUC<T>(op2),
			      max(Time(), op2.observationTime()),
			      op2.printObservationTime());
#endif
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator* (const WithUnknown<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
  return op2 * op1;
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> operator/ (const WithUnknown<T>& op1, const AngleDegreesUCO<T, Time>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return AngleDegreesUCO<T, Time>(op1 / AngleDegreesUC<T>(op2),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return AngleDegreesUCO<T, Time>(op1 / AngleDegreesUC<T>(op2),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<typename T, class Time>
std::ostream& operator<<(std::ostream& os, const AngleDegreesUCO<T, Time>& obj)
{
  if (obj.printObservationTime())
    return os << AngleDegreesUC<T>(obj) << " observed at " << obj.getObservationTime();
  else
    return os << AngleDegreesUC<T>(obj);
}


#ifdef RS_TRIG_IN_DEG


template<typename T, class Time>
inline AngleDegreesUCO<T, Time> ASin(const WithUCO<T, Time>& op)
{
  //  if ( op.isUnknown() )
  //    return AngleDegreesUCO<T, Time>();
  //  else
    return AngleDegreesUCO<T, Time>(ASin(WithUC<T>(op)));
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> ACos(const WithUCO<T, Time>& op)
{
  //  if ( op.isUnknown() )
  //    return AngleDegreesUCO<T, Time>();
  //  else
    return AngleDegreesUCO<T, Time>(ACos(WithUC<T>(op)));
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> ATan(const WithUCO<T, Time>& op)
{
  //  if ( op.isUnknown() )
  //    return AngleDegreesUCO<T, Time>();
  //  else
    return AngleDegreesUCO<T, Time>(ATan(WithUC<T>(op)));
}

template<typename T, class Time>
inline AngleDegreesUCO<T, Time> ATan2(const WithUCO<T, Time>& op1, const WithUCO<T, Time>& op2)
{
  //  if ( op1.isUnknown() ||
  //       op2.isUnknown() )
  //    return AngleDegreesUCO<T, Time>();
  //  else
    return AngleDegreesUCO<T, Time>(ATan2(WithUC<T>(op1), WithUC<T>(op2)));
}

#endif // RS_TRIG_IN_DEG


RS_END_NAMESPACE


#endif // ANGLEDEGREESUCO_H
