/* -*- Mode: C++ -*-
 * $Id: CoachCommand.h,v 1.1 2001/11/15 16:48:31 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoachCommand.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2001-11-15
 * Last modified by $Author: frehe $ $Date: 2001/11/15 16:48:31 $
 *
 *
 * HISTORY
 *
 * $Log: CoachCommand.h,v $
 * Revision 1.1  2001/11/15 16:48:31  frehe
 * Added a coach command.
 *
 */

/**
 * @file CoachCommand.h
 * Include file for the class CoachCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2001/11/15 16:48:31 $
 */


#ifndef COACHCOMMAND_H
#define COACHCOMMAND_H


#include "ServerCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class CoachCommand CoachCommand.h
 * Class used to represent a generic coach command
 * defined in RoboSoc.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2001/11/15 16:48:31 $
 */
class CoachCommand : public ServerCommand {
public:
  CoachCommand(const std::string& cmd);
  CoachCommand(const CoachCommand& obj);
  virtual ~CoachCommand();

  CoachCommand& operator =(const CoachCommand& obj);

  
  /** Returns @c true iff only one command of this type can be
    sent each simulator cycle. */
  virtual bool onlyOneEachCycle() const;

  
  /** Returns @c true since it is an immediate command. */
  virtual bool isImmediateCommand() const;


  /** Returns the server string that will execute the CoachCommand
    at the soccer server. */
  virtual const std::string serverString() const;

  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual CoachCommand* clone() const;

  
protected:
  void copyCommand(const CoachCommand& obj);


 private:
  /** The actual commands. */
  std::string command;
};


RS_END_NAMESPACE


#endif // !COACHCOMMAND_H
