/* -*- Mode: C++ -*-
 * $Id: TVector.h,v 1.1 2001/09/17 21:31:53 andbo928 Exp $
 *
 * IDENTIFICATION
 *
 * Filename: WorldStateView.cc
 * Unitname: Framework/view
 * $Revision: 1.1 $
 * Created by: Mikael Brnnstrm, Andreas Sigfridsson, Andreas Bckert
 * Last modified by $Author: andbo928 $ $Date: 2001/09/17 21:31:53 $
 *
 *
 * HISTORY
 *
 * $Log: TVector.h,v $
 * Revision 1.1  2001/09/17 21:31:53  andbo928
 * Imported from FuzzyFoo source.
 *
 * Revision 1.13  2001/06/05 20:33:53  mikbr212
 * Fixed doxygen-stubs for all classes
 *
 * Revision 1.12  2001/04/14 08:40:00  mikbr212
 * removed warnings
 *
 * Revision 1.11  2001/04/13 17:33:33  mikbr212
 * added +=, -=, *=, /=
 *
 * Revision 1.10  2001/04/13 08:17:28  andbo928
 * Removed DOS-style returns.
 *
 * Revision 1.9  2001/04/12 16:02:22  mikbr212
 * Added icc and static stuff
 *
 *
 */

/**
 * @file TVector.h
 * Definition file for the class TVector.
 *
 * $Revision: 1.1 $
 * $Author: andbo928 $
 * $Date: 2001/09/17 21:31:53 $
 */

#ifndef TVECTOR_H
#define TVECTOR_H
#include "library_types.h"
#include "library_types_with_unknown.h"

RS_BEGIN_NAMESPACE

/**
 * @class TVector TVector.h
 * Text...
 */

class TVector
{
public:
  TVector();
  TVector(double _x, double _y);
  TVector(double _x, double _y, double _c);


  // Static methods
  static TVector TPolarVector(double r, double a);
  static double degToRad(double deg);
  static double radToDeg(double rad);

  // Compatibility constructors.
  TVector(const Vector& other);
  TVector(const VectorUnknown& other);
  TVector(const VectorUC& other);
  TVector(const VectorUCO& other);
  TVector(const Point& other);
  TVector(const PointUnknown& other);
  TVector(const PointUC& other);
  TVector(const PointUCO& other);

  TVector operator+(const TVector& other) const;
  TVector operator-(const TVector& other) const;
  TVector operator-() const;
  double operator*(const TVector& other) const;
  TVector operator*(double d) const;
  friend TVector operator*(double d, const TVector& other);

  TVector operator/(double d) const;
  TVector normalized();

  TVector& operator +=(const TVector&);
  TVector& operator -=(const TVector&);

  TVector& operator *=(const double&);
  TVector& operator /=(const double&);

  bool confCompare(const TVector& other) const;
  TVector confSelect(const TVector& other) const;

  // Get/Set functions
  double getX() const;
  const TVector& setX(double _x);
  double getY() const;
  const TVector& setY(double _y);
  double getC() const;
  const TVector& setC(double _c);


  const TVector& setPolar(double r, double a);
  const TVector& rotate(double a);

  double getLength() const;
  const TVector& setLength(double len);
  double getAngle() const;
  const TVector& setAngle(double angle);

  bool operator==(const TVector& other) const;

private:
  double x;
  double y;
  double c; // Confidence.

protected:

};

std::ostream& operator<<(std::ostream&, const TVector& vec);


RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "TVector.icc"
#endif

#endif // TVECTOR_H 

